#################################################################################
# Created       : Martin Foster
# Modified      : 08/23/2000
#################################################################################
#
# Comm - Module part of Webchat designed to prompt and handle posts.
# Copyright (C) 2000  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : mfoster@redwhite.com
#       Address : Suite 209 - 355 5th Ave NE
#                 Calgary, Alberta
#                 T2E 0K9
#
#################################################################################

package Webchat::Comm;						# Pacakace name

use strict;							# Strict variable enforcement

use Webchat::Database;						# Database handler
use Webchat::Date;						# Date hander
use Webchat::Dbm; 						# DBM Addition
use Webchat::Login;						# Login functionality
use Webchat::Param;						# Parameter control


#################################################################################
# Sub-Routines
#################################################################################

#####################
# Comm
#
# Subroutine primarily charged with displaying input area.   This will also call
# CommHandler which will handle the requests to determin the need for a new
# database entry

sub Comm							# Database, CGI, Parameter list
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# CGI handle
	my $param = shift;					# Parameter list

	my $choice = shift;					# Choice of the matter
	my $java;						# Java options

	my $inline;						# Inline argument list
	my $url = $cgi->url();					# Url of script

	my $puppet;						# Puppet name
	my $color;						# Colour default
	my $size;						# Size default

	my @color;						# Colours
	my @size;						# Size fonts
	my @char;						# Character

	my %chat;						# Portal hash


	#####################
	# Program area

	# Link DBM to hash
	Webchat::Dbm::DBMOpenChat(\%chat);	

	# Populate arrays
	OptionSplit(\@color, $chat{'OptTextColour'});
	OptionSplit(\@size,   $chat{'OptTextSize'});
	Webchat::Database::GetListPuppetYours($Database, \@char, $param->{'USER'});
	

	# Cleanup for embed
	delete $param->{'POST'};

	# Define inline and puppet
	$inline = Webchat::Param::EmbedInline($Database, $cgi, %{$param});
	$puppet = $param->{'CHAR'};

	# Retreive size and color
	($size, $color) = Webchat::Database::DataGetFont($Database, $puppet, $color[0]);

	
	# Independent HTML
	if ($choice != 0)
	{

		# Print file and HTML header
		print $cgi->header(-target=>'comm');
		print $cgi->start_html('Chat subsection');
	}

	# With or without javascript as prescribed
	# Determine need for javascript
	($java) = Webchat::Database::DataGetJavascript($Database, $param->{'USER'});

	# Print if javascript is desired
	if ($java eq 'yes')
	{ Webchat::Database::DocumentGetJavascript($Database); }

	# HTML
	# Form
	if ($choice == 1)					# Framed
	{ 
		print $cgi->start_form(-encoding=>'application/x-www-form-urlencoded',
			-method=>'POST',
			-action=>"$url",
			-name=>'Comm');
	} 
	else							# Dependant/streaming version
	{ print $cgi->start_form(); }

	# Top and beautifying code
	if ($java eq 'yes')					# Scrolling
	{
		print "<CENTER>\n<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=2 BGCOLOR=\"$chat{'SetBGColor'}\" WIDTH=500>\n";
		print "\t<TR>\n\t\t<TD COLSPAN=\"4\"><A HREF=\"\" ONCLICK=\"return false;\" ONMOUSEOVER=\"ScrollerOn()\" ONMOUSEOUT=\"ScrollerOff()\">$chat{'SetTop'}</A></TD>\n\t</TR>\n";
	}
	else							# Absence of scrolling
	{
		print "<CENTER>\n<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=2 BGCOLOR=\"$chat{'SetBGColor'}\" WIDTH=500>\n";
		print "\t<TR>\n\t\t<TD COLSPAN=\"4\">$chat{'SetTop'}</TD>\n\t</TR>\n";
	}
	

	# Posting field and side spacers
	print "\t<TR>\t\t<TD ROWSPAN=\"4\" WIDTH=\"5\"><FONT COLOR=\"$chat{'SetBGColor'}\">.</FONT></TD>\n";
	print "\t\t<TD COLSPAN=\"2\"><TEXTAREA NAME=\"POST\", ROWS=\"3\", COLS=\"55\", WRAP=\"SOFT\"></TEXTAREA></TD>\n";
	print "\t\t<TD ROWSPAN=\"4\" WIDTH=\"5\"><FONT COLOR=\"$chat{'SetBGColor'}\">.</FONT></TD>\n\t</TR>\n";

	# Puppet submit
	print "\t<TR>\n\t\t<TD ROWSPAN=\"3\">", $cgi->submit($chat{'TxtSubmitButton'}),"</TD>\n";


	# Puppet selection
	print "\t\t<TD>$chat{'TagPuppet'}", $cgi->popup_menu('CHAR', \@char, $puppet), "</TD>\n\t</TR>\n";

			
	# Text modifications
	print "\t<TR>\n\t\t<TD>$chat{'TagTextColour'}",  $cgi->popup_menu('COLOR', \@color, $color), "</TD>\n\t</TR>\n";
	print "\t<TR>\n\t\t<TD>$chat{'TagTextSize'}", $cgi->popup_menu('SIZE', \@size, $size), "</TD>\n\t</TR>\n";


	# Links
	if ($java eq 'yes')
	{
		# Javascript enabled
		print "\t<TR>\n\t\t<TD COLSPAN=\"4\">",
 		     "<A HREF=\"javascript:popup_window('$chat{'LnkPuppet'}$inline',1)\">$chat{'TagLnkPuppet'}</A>", 
		     "<A HREF=\"javascript:popup_window('$chat{'LnkIgnore'}$inline',1)\">$chat{'TagLnkIgnore'}</A>",
		     "<A HREF=\"javascript:popup_window('$chat{'LnkWho'}$inline',1)\">$chat{'TagLnkWho'}</A>",
		     "<A HREF=\"javascript:popup_window('$chat{'LnkPrivate'}$inline',1)\">$chat{'TagLnkPrivate'}</A>",
	     	     "</TD>\n\t</TR>\n"; 
	}
	else
	{
		# Self containted window
		print "\t<TR>\n\t\t<TD COLSPAN=\"4\">",
		      "<A HREF=\"$chat{'LnkPuppet'}$inline\" TARGET=\"_TOP\">$chat{'TagLnkPuppet'}</A>", 
		      "<A HREF=\"$chat{'LnkIgnore'}$inline\" TARGET=\"_TOP\">$chat{'TagLnkIgnore'}</A>",
		      "<A HREF=\"$chat{'LnkWho'}$inline\" TARGET=\"_TOP\">$chat{'TagLnkWho'}</A>", 
		      "<A HREF=\"$chat{'LnkPrivate'}$inline\" TARGET=\"_TOP\">$chat{'TagLnkPrivate'}</A>",
		      "</TD>\n\t</TR>\n"; 
	}

	# Table Bottom
	print "</TABLE>\n</CENTER>\n";

	# Embed Main or Comm
	if ($choice == 0)					# For framed chat
	{  Webchat::Param::EmbedNormal($Database, $cgi, %{$param});  }
	elsif ($choice == 1)					# For framed chat
	{  Webchat::Param::EmbedNormal($Database, $cgi, %{$param}, 'MAIN', 'true');
	   print "\t", $cgi->hidden('NUM', $param->{'NUM'}), "\n"; }
	elsif ($choice == 2)					# For streaming
	{   Webchat::Param::EmbedNormal($Database, $cgi, %{$param}, 'COMM', 'true'); }

	# End of form
	print $cgi->end_form(), "\n";

	# Print end of html if needed
	if (($choice == 0) || ($choice eq 'no'))
	{ print $cgi->end_html(); }

	# Unlink DBM to hash
	Webchat::Dbm::DBMClose(\%chat);	
}

# Alias for dependency
sub CommDependant
{ Comm(shift, shift, shift, 0); }

# Alias for independence
sub CommIndependant
{ Comm(shift, shift, shift, 1); }

# Alias for independence
sub CommStream
{ Comm(shift, shift, shift, 2); }


#####################
# DataWrite
#
# Encharged with properly handling information and actually making posts
# once formatted inserting them into the database.

sub DataWrite							# Database, param
{
        #####################
        # Data members
	my $Database = shift;					# Database get
	my $cgi = shift;					# Common gateway interface
	my $param = shift;					# Parameters

	my $hex;						# Hex colour
	my %chat;						# Chat hash


        #####################
        # Data members

	# Assign to target if undefined
	unless (defined($param->{'TARGET'}))
	{ $param->{'TARGET'} = 'all'; }

	# Only do following if posts exist
	if ((defined($param->{'POST'})) && (($param->{'POST'} =~ tr/a-zA-Z0-9*:()//) > 1))
	{
		# Link DBM to hash
		Webchat::Dbm::DBMOpenChat(\%chat);	

		# Determin font information
		$hex   = OptionFindColor($chat{'OptTextColour'}, $chat{'OptTextColourHex'}, $param->{'COLOR'});

		# Update database entry
		$Database->do("UPDATE Puppet SET PuppetTextColour=?, PuppetTextSize=? WHERE PuppetName=?",
			{}, $param->{'COLOR'}, $param->{'SIZE'}, $param->{'CHAR'});

		# Handle posts
		PostHandler($Database, $cgi, $param, \%chat); 		

		# Unlink DBM to hash
		Webchat::Dbm::DBMClose(\%chat);	
	}
	
	# No post simply post for timestamp purposes
	else
	{
		$Database->do("INSERT INTO Post VALUES(NULL, ?, ?, 'system', ?, NULL, NULL, NULL)",
			{}, $param->{'ROOM'}, $param->{'CHAR'}, time);

	}
}

#####################
# GetDate
#
# Returns a cleaner version of time in the form of a string that would
# normally not be acheived by plainly calling the gmtime function.

sub GetDate
{
	#####################
	# Data members
	my $time = shift;					# Time


	#####################
	# Program area

	# Retreive time
	my ($sec, $min, $hour, $mday, $mon, $year) = localtime($time);

	# Massage values
	$sec  = ($sec    < 10) ? "0$sec"  : $sec;
	$min  = ($min    < 10) ? "0$min"  : $min;
	$hour = ($hour   < 10) ? "0$hour" : $hour;
	$mday = ($mday   < 10) ? "0$mday" : $mday;
	$mon  = (++$mon  < 10) ? "0$mon"  : $mon;

	# Handle year
	$year += 1900;

	# Return appropriate value
	return "${mon}/${mday}/${year} ${hour}:${min}:${sec}";
}


#####################
# OptionSplit
#
# Splits options list into seperate elements of array

sub OptionSplit
{
        #####################
        # Data members

        my $array = shift;                                      # Reference to array
        my $data = shift;                                       # Retreived line


        #####################
        # Program area
        @{ $array } = split(/:/, $data);                        # Split line
}


#####################
# OptionFindColor
#
# Splits options lists and determins hex value of post

sub OptionFindColor						# Colors, Hex, Value
{
	#####################
	# Data members

	my $colors = shift; 					# String of colours
	my $hex = shift;					# String of hex colours
	my $value = shift;					# Value to seach for

	my @colors;						# Array of colors
	my @hex;						# Array of hex values


	#####################
	# Program area

	# Retreive colours and hex
	OptionSplit(\@colors, $colors);
	OptionSplit(\@hex, $hex);	

	# Find lenght of colors
	$_ = @colors;

	# Determine proper index
	for (my $count; $count < $_; $count++)
	{ 
		# And return once found
		if ($colors[$count] eq $value)
		{ return $hex[$count]; }
	}
}


#####################
# Post Handler
#
# Depending on rules will handle the formatting of the three styles of posts
# filter language and HTML if needed or specified.

sub PostHandler
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# Common gateway interface
	my $param = shift;					# Reference to parameters
	my $chat = shift;					# Reference to chat

	my $full;						# Full formating
	my $imageless;						# No images
	my $spartan;						# Spartan formatting

	my $title;						# Title
	my $tag;						# Tagline
	my $inline;						# Inline parameters
	my $bio;						# Bio link

	my $start;						# Start of table
	my $end;						# End of table

	my $time = Webchat::Date::GetDate(time);		# Date and Time

	my @realm;						# Realm information
	my @puppet;						# Puppet information
	my @lang;						# Language array
	my @realmpic;						# Ream picture restriction

	my %chat;						# Chat DBM


	#####################
	# Program area

	# Retreive puppet and realminformation
	@puppet = Webchat::Database::DataGetPuppetInfo($Database, $param->{'CHAR'});
	@realm = Webchat::Database::DataGetRealmInfo($Database, $param->{'ROOM'});	

	# Language filtering
	if ($realm[0] eq 'restricted')
	{
		# Retreive words
		# Prepare and execute
		my $Statement = $Database->prepare("SELECT WordSpelling FROM WordRestriction WHERE RealmName=?");
		$Statement->execute($param->{'ROOM'});

		# Retreive and place in array
		while ($_ = $Statement->fetchrow())
		{ push(@lang, $_); }
		$Statement->finish;

		
		# Run though array of words and filter post
		$_ = @lang;					# Get lenght		
		for (my $count=0; $count < $_; $count++)
		{ $param->{'POST'} =~ s/$lang[$count]/****/gsi; }
	}

	# Bio link
	$inline = Webchat::Param::EmbedInline($Database, $cgi, 'CHAR', $param->{'CHAR'});
	$bio = "<A HREF=\"$chat->{'LnkBio'}$inline\" TARGET=\"_TOP\">$chat->{'TagPuppetBio'}</A>";

	# Format title
	if (defined($puppet[6]))
	{ 
		# Verify if NAME attribute exists
		if ($puppet[6] =~ /NAME/)
		{
			$puppet[6] =~ s/NAME/$param->{'CHAR'}/g;
			$title = $puppet[6];
		}

		# Otherwise prepend name
		else
		{ $title = "$param->{'CHAR'} $title"; }
	}
	

	# Assign chacter name to title
	else
	{ $title = $param->{'CHAR'}; }	

	# Create formatted line
	$title = "$bio <B><FONT SIZE=\"$chat->{'SetTitleSize'}\">$title</FONT></B>";

	# Create tagline
	unless (($param->{'TARGET'} ne 'all') && ($param->{'TARGET'} ne 'system'))
	{ $tag   = "(<I>$puppet[2]</I>) ($time)"; }
	else
	{ $tag   = "(<I>$puppet[2]</I>) ($chat->{'TagPuppetPMFrom'}: $param->{'CHAR'}) ($chat->{'TagPuppetPMTo'}: $param->{'OTARGET'}) ($time)"; }
	
	
	# Filter HTML if option is selected
	if ($realm[1] eq 'restricted')
	{ $param->{'POST'} =~ s/<[^>]*>//gs; }


	# Determine colour
	my $hex = OptionFindColor($chat->{'OptTextColour'}, $chat->{'OptTextColourHex'}, $param->{'COLOR'});

	# Add colors
	$param->{'POST'} = "<FONT SIZE=\"$param->{'SIZE'}\" COLOR=\"$hex\">$param->{'POST'}</FONT>";


	# If images use following
	if (defined($puppet[3]))
	{
		# Retreive image size information
		@realmpic = Webchat::Database::DataGetRealmImageInfo($Database, $param->{'ROOM'});
		
		# Format image format accordingly
		if ((($realmpic[0] != 0) && ($realmpic[1] != 0)) && 
			(($puppet[4] > $realmpic[0]) || ($puppet[5] > $realmpic[1]))) 
		{ $full = "<A HREF=\"$puppet[3]\" TARGET=\"_TOP\">$chat->{'TagLnkImage'}</A><BR>$title $tag<BR>$param->{'POST'}"; }
		else
		{ $full = "<IMG SRC=\"$puppet[3]\" HEIGHT=\"$puppet[4]\" WIDTH=\"$puppet[5]\" ALIGN=\"left\" BORDER=\"0\">$title $tag<BR>$param->{'POST'}"; }

		# Format non imaged format
		$imageless = "$title $tag<BR>$param->{'POST'}";
	}
	else
	{
		# Non imaged format which is assigned to full if no image is found
		$imageless = "$title $tag<BR>$param->{'POST'}";
		$full = $imageless;
	}

	# Create spartan format
	$param->{'POST'} =~ s/<[^>]*>//gs;
	$spartan = "$title $tag<BR>$param->{'POST'}<BR><BR>";

	# Embed all formats in tables
	unless (($param->{'TARGET'} ne 'all') && ($param->{'TARGET'} ne 'system'))
	{
		# For normal posts
		$start = "<TABLE>\n<TR>\n\t\t<TD>";
		$end   = "</TD>\n\t</TR>\n</TABLE>\n";
	}
	else
	{
		# Embed for private posts
		$start = "<TABLE BORDER=\"3\">\n<TR>\n\t\t<TD>";
		$end   = "</TD>\n\t</TR>\n</TABLE>\n";
	}

	# Convert \n to a useable character	
	$full      =~ s/\n/<BR>/gs;
	$imageless =~ s/\n/<BR>/gs;
	$spartan   =~ s/\n/<BR>/gs;

	# Assembly
	$full      = "$start$full$end";
	$imageless = "$start$imageless$end";
	$spartan   = "$start$spartan$end";


	# Insert into database
	$Database->do("INSERT INTO Post 
		(RealmName,
		 PuppetName,
		 PostTo,
		 PostTimestamp,
		 PostFullFormat,
		 PostImagelessFormat,
		 PostPartialFormat)		 
		VALUES(?, ?, ?, ?, ?, ?, ?)", {}, 
	 $param->{'ROOM'}, 
	 $param->{'CHAR'}, 
	 $param->{'TARGET'}, 
	 time, 
	 $full, 
	 $imageless, 
	 $spartan);

	# Insert message to be returned to poster
	if (($param->{'TARGET'} ne 'all') && ($param->{'TARGET'} ne 'system'))
	{
		$Database->do("INSERT INTO Post 
			(RealmName,
			 PuppetName,
			 PostTo,
			 PostTimestamp,
			 PostFullFormat,
			 PostImagelessFormat,
			 PostPartialFormat)
			VALUES(?, ?, ?, ?, ?, ?, ?)", {}, 
		 $param->{'ROOM'}, 
		 $param->{'CHAR'}, 
		 $param->{'SOURCE'}, 
		 time, 
		 $full, 
		 $imageless, 
		 $spartan);
	}
}

1;								# Return true
