#################################################################################
# Created       : Martin Foster
# Modified      : 05/23/2000
#################################################################################
#
# Crypto - Module hiding the complexity behind cryptographic uses in Ethereal 
# Copyright (C) 2000  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : mfoster@redwhite.com
#       Address : Suite 209 - 355 5th Ave NE
#                 Calgary, Alberta
#                 T2E 0K9
#
#################################################################################
package Webchat::Crypto;					# Module name


use strict;							# Strict variable enforcement

use Compress::Zlib;
use Crypt::CBC;							# Cryptographic cipher block chaining mode
use Digest::MD5 qw(md5_hex);					# MD5 digest cryptographic algorithm	


#################################################################################
# Data Members
#################################################################################

my $pass = 'CHANGE';						# Password


#################################################################################
# Sub-Routines
#################################################################################

#####################
# get Cipher Text
# 
# Uses the cryptographic Cipher Block Chainining (CBC) module to encrypt the
# given parameter.

sub getCipherText						# Plain text, salt
{
	#####################
	# Data members
	my $text 	= shift;				# Retreived text
	my $cbc;						# CBC handle


	#####################
	# Program area

	# Create CBC handle
	$cbc = new Crypt::CBC($pass, 'Blowfish');

	# Return ciphertext
	return $cbc->encrypt_hex($text);		
}


#####################
# getPlainText
# 
# Uses the cryptographic Cipher Block Chainining (CBC) module to decrypt the
# given parameter.

sub getPlainText						# Cipher text, salt
{
	#####################
	# Data members
	my $text 	= shift;				# Retreived text

	my $cbc;						# CBC handle

	#####################
	# Program area

	# Create CBC handle
	$cbc = new Crypt::CBC($pass, 'Blowfish_PP');

	# Return ciphertext
	return $cbc->decrypt_hex($text);		
}


#####################
# decryptParam
#
# Will take a reference to a hash, seperate it together using : as the seperator
# then calculate a MD5 hash compare and return 1 for good.

sub decryptParam
{
	#####################
	# Data members
	my $crypt	= shift;				# Crypted parameter list
	my $hash	= shift;				# Hash value

	my $string;						# String to work on
	my $md5;						# MD5 hash value


	#####################
	# Program area

	# Decrypt and decompress line
	$crypt  = getPlainText($crypt);
	$crypt  = uncompress($crypt) || die("Unable to uncompress parameter");

	# Extract the parameter list and MD5 hash
	$md5    = substr($crypt, -32, 32);
	$string = substr($crypt, 0, -33);

	# Compare hashes
	if ($md5 eq md5_hex($string))
	{
		# Split and return
		%{$hash} = split(/\|/, $string);
		return 1;
	}
	
	return 0;						# Return false if reached
}


#####################
# encryptParam
#
# Will take a flatened hash (key/pair), join it together using : as the seperator
# calculate a MD5 hash then encrypt the entire line using the provided key.

sub encryptParam
{
	#####################
	# Data members
	my $string;						# String to work on


	#####################
	# Program area

	$string = join('|', @_);				# Concatenante to string
	$string = join('', $string, '|', md5_hex($string));	# Append MD5 value to end
	$string = compress($string) || die("Cannot compress");  # Compress string

	return getCipherText($string);				# Encrypt total and return
#	return $string;
}


#####################
# makeSalt
#
# Produces a salt using rand() and the Digest::MD5 hash.  Which once truncated to
# fourteen characters will be returned to parents function.

sub makeSalt
{
	#####################
	# Data members
	my $salt;						# Salt value

	#####################
	# Program area	
	
	# Generate hash, truncate then return
	$salt = md5_hex(rand());
	return substr($salt, 0, 14);
}

1;								# Return true
